--WEAPONS.LUA--
--Push Squad Weapon Definitions--

--Prime Harpoon (Prime)--
Prime_Harpoon = Skill:new{
	Name = "Prime Harpoon",
	Class = "Prime",
	Icon = "weapons/prime_spear.png",
	Description = "Stab multiple tiles and pull the furthest hit tile.",
	Explosion = "",
	Range = 2, 
	PathSize = 2,
	Damage = 1,
	Push = 1,
	Acid = 0,
	PowerCost = 1,
	Upgrades = 2,
	UpgradeCost = { 2 , 3 },
	UpgradeList = {"+1 Range", "+2 Damage"},
	LaunchSound = "/weapons/sword",
	TipImage = {
		Unit = Point(2,3),
		Enemy = Point(2,2),
		Enemy2 = Point(2,1),
		Target = Point(2,1)
	}
}

function Prime_Harpoon:GetTargetArea(point)
	local ret = PointList()
	for i = DIR_START, DIR_END do
		for k = 1, self.PathSize do
			local curr = DIR_VECTORS[i]*k + point
			ret:push_back(curr)
			if not Board:IsValid(curr) then --or Board:GetTerrain(curr) == TERRAIN_MOUNTAIN then
				break
			end
		end
	end
	
	return ret
end
				
function Prime_Harpoon:GetSkillEffect(p1, p2)
	local ret = SkillEffect()
	local direction = GetDirection(p2 - p1)
	local pull = GetDirection(p1 - p2)
	local distance = p1:Manhattan(p2)

	for i = 1, distance do
		local push = (i == distance) and pull*self.Push or DIR_NONE
		local damage = SpaceDamage(p1 + DIR_VECTORS[direction]*i,self.Damage, push)
		if i == distance then
			damage.iAcid = self.Acid
		end
		if i == 1 then
			damage.sAnimation = "explospear"..distance.."_"..direction
		end
		ret:AddDamage(damage)
	end

	return ret
end	

Prime_Harpoon_B = Prime_Harpoon:new{
	UpgradeDescription = "Increase damage by 2.",
	Damage = 3,
}

Prime_Harpoon_A = Prime_Harpoon:new{
	UpgradeDescription = "Increase range by 1.",
	PathSize = 3, 
	Range = 3,
	TipImage = {
		Unit = Point(2,4),
		Enemy = Point(2,2),
		Enemy2 = Point(2,1),
		Target = Point(2,1)
	}
}

Prime_Harpoon_AB = Prime_Harpoon:new{
	PathSize = 3, 
	Range = 3,
	Damage = 3,
	TipImage = {
		Unit = Point(2,4),
		Enemy = Point(2,2),
		Enemy2 = Point(2,1),
		Target = Point(2,1)
	}
}

--Vortex Artillery (Ranged)--
Push_Artillery = LineArtillery:new{
	Name = "Vortex Artillery",
	Class = "Ranged",
	Icon = "weapons/science_gravwell.png",
	Description = "Pull adjacent targets onto the centre. Targets are pulled clock-wise from the back.",
	Sound = "",
	Explosion = "",
	PowerCost = 1,
	BounceAmount = 1,
	BigSize = 0,
	Damage = 0,
		ArtilleryStart = 2,
		ArtillerySize = 8,
	BigSize = 0,
	LaunchSound = "/weapons/gravwell",
	UpShot = "effects/shotup_guided_missile.png",
	Upgrades = 2,
	UpgradeCost = {2,2},
	UpgradeList = {"+2 Area", "+2 Damage"},
	TipImage = {
		Unit = Point(2,3),
		Enemy = Point(2,1),
		Enemy2 = Point(3,1),
		Enemy3 = Point(1,1),
		Target = Point(2,1)
	}
}
					
function Push_Artillery:GetSkillEffect(p1,p2)
	local ret = SkillEffect()
	local dir = GetDirection(p2 - p1)
	local dir2 = GetDirection(p1 - p2)
	local damage = SpaceDamage(p2, self.Damage)
	local damagepush = SpaceDamage(p2 + DIR_VECTORS[(dir+1)%4], 0, (dir-1)%4)
	
	ret:AddBounce(p1, -2)
	ret:AddArtillery(damage,"effects/shotup_ice.png")
	ret:AddBounce(p2, self.BounceAmount)

	if self.BigSize == 1 then
		--North--
		damagepush = SpaceDamage(p2 + DIR_VECTORS[dir], 0, dir2)
		damagepush.sAnimation = "airpush_"..dir2
		ret:AddDamage(damagepush)		
		ret:AddDelay(0.2)
	end
	--East--
	damagepush = SpaceDamage(p2 + DIR_VECTORS[(dir+1)%4], 0, (dir-1)%4)
	damagepush.sAnimation = "airpush_"..((dir-1)%4)
	ret:AddDamage(damagepush)
	ret:AddDelay(0.2)
	
	if self.BigSize == 1 then
		--South--
		damagepush = SpaceDamage(p2 - DIR_VECTORS[dir], 0, dir)
		damagepush.sAnimation = "airpush_"..dir
		ret:AddDamage(damagepush)	
		ret:AddDelay(0.2)
	end
	
	--West--
	damagepush = SpaceDamage(p2 + DIR_VECTORS[(dir-1)%4], 0, (dir+1)%4)
	damagepush.sAnimation = "airpush_"..((dir+1)%4)
	ret:AddDamage(damagepush)	
	
	
	return ret
end

Push_Artillery_A = Push_Artillery:new{
	UpgradeDescription = "Increases area by 2 tiles.",
	BigSize = 1,
	TipImage = {
		Unit = Point(2,3),
		Enemy = Point(2,1),
		Enemy2 = Point(3,1),
		Enemy3 = Point(1,1),
		Enemy4 = Point(2,2),
		Enemy5 = Point(2,0),
		Target = Point(2,1)
	}
}

Push_Artillery_B = Push_Artillery:new{
	UpgradeDescription = "Increases damage by 2.",
	Damage = 2,
}

Push_Artillery_AB = Push_Artillery:new{
	BigSize = 1,
	Damage = 2,
	TipImage = {
		Unit = Point(2,3),
		Enemy = Point(2,1),
		Enemy2 = Point(3,1),
		Enemy3 = Point(1,1),
		Enemy4 = Point(2,2),
		Enemy5 = Point(2,0),
		Target = Point(2,1)
	}
}

--Kick Off Charge (Brute)--
Rock_Charge = Brute_Beetle:new{
	Name = "Kick-off Charge",
	Class = "Brute",
	Portrait = "",
	Icon = "weapons/brute_beetle.png",
	Description = "Fly in a line and slam into the target, pushing it. Kick-off from the target behind you to do extra damage.",
	Rarity = 3,
	Explosion = "",
	Push = 1,--TOOLTIP HELPER
	Fly = 1,
	Damage = 1,
	BuildingDamage = true,
	SelfDamage = 0,
	DamageKick = 1,
	DamageBuilding = 0,
	PathSize = INT_MAX,
	PowerCost = 1,
	Upgrades = 2,
	UpgradeCost = {1,3},
	UpgradeList = {"Building Kick-Off", "+2 Kick Damage"},
	LaunchSound = "/weapons/charge",
	ImpactSound = "/weapons/charge_impact",
	TipImage = {
		Unit = Point(2,2),
		Enemy = Point(4,2),
		Enemy2 = Point(1,2),
		Target = Point(4,2),
	}
}

function Rock_Charge:GetSkillEffect(p1,p2)
	local ret = SkillEffect()
	local direction = GetDirection(p2 - p1)
	local reverse = GetDirection(p1 - p2)

	local pathing = PATH_PROJECTILE
	if self.Fly == 0 then pathing = Pawn:GetPathProf() end

	local doDamage = true
	local target = GetProjectileEnd(p1,p2,pathing)
	local distance = p1:Manhattan(target)
	local kickoff = 0
	local buildingkickoff = 0
	
	if not Board:IsBlocked(target,pathing) then -- dont attack an empty edge square, just run to the edge
		doDamage = false
		target = target + DIR_VECTORS[direction]
	end
	
	local smoke = SpaceDamage(p1 - DIR_VECTORS[direction], self.Damage, reverse)
	
	if not self.BuildingDamage and Board:IsBuilding(p1 - DIR_VECTORS[direction]) then	
		smoke.iDamage = 0
	end	
	if Board:IsBuilding(p1 - DIR_VECTORS[direction]) then	
		buildingkickoff = self.DamageBuilding + self.DamageKick
	end
	if Board:GetPawn(p1 - DIR_VECTORS[direction]) then
		kickoff = self.DamageKick
	end

	ret:AddDamage(smoke)
	
	local damage = SpaceDamage(target, self.Damage + kickoff + buildingkickoff, direction)
	damage.sAnimation = "ExploAir2"
	damage.sSound = self.ImpactSound
	
	if distance == 1 and doDamage then
		ret:AddMelee(p1,damage, NO_DELAY)
		if doDamage then ret:AddDamage(SpaceDamage( target - DIR_VECTORS[direction] , self.SelfDamage)) end
	else
		ret:AddCharge(Board:GetSimplePath(p1, target - DIR_VECTORS[direction]), NO_DELAY)--FULL_DELAY)

		local temp = p1 
		while temp ~= target  do 
			ret:AddBounce(temp,-3)
			temp = temp + DIR_VECTORS[direction]
			if temp ~= target then
				ret:AddDelay(0.06)
			end
		end
		
		if doDamage then
			ret:AddDamage(damage)
			ret:AddDamage(SpaceDamage( target - DIR_VECTORS[direction] , self.SelfDamage))
		end
	
	end
	

	return ret
end

Rock_Charge_A = Rock_Charge:new{
	UpgradeDescription = "Kick off a building to do extra damage. Buildings are no longer damaged by a kick off.",
	BuildingDamage = false,
	DamageBuilding = 1,
	TipImage = {
		Unit = Point(2,2),
		Enemy = Point(4,2),
		Building = Point(1,2),
		Target = Point(4,2),
	}
}

Rock_Charge_B = Rock_Charge:new{
	UpgradeDescription = "Increase kick-off damage by 2.",
	DamageKick = 3	
}

Rock_Charge_AB = Rock_Charge:new{
	BuildingDamage = false,
	DamageBuilding = 1,
	DamageKick = 3
}


--Testing--
Vortex_Artillery = LineArtillery:new{
	Name = "Vortex Artillery",
	Class = "Ranged",
	Icon = "weapons/science_gravwell.png",
	Description = "Pull adjacent targets clockwise around the centre target.",
	Sound = "",
	Explosion = "",
	PowerCost = 1,
	BounceAmount = 1,
	BigSize = 0,
	Damage = 1,
	DamageOuter = 0,
	CentreSmoke = false,
		ArtilleryStart = 2,
		ArtillerySize = 8,
	BigSize = 0,
	LaunchSound = "/weapons/gravwell",
	UpShot = "effects/shotup_guided_missile.png",
	Upgrades = 2,
	UpgradeCost = {2,2},
	UpgradeList = {"+2 Outer Damage", "Centre Smoke"},
	TipImage = {
		Unit = Point(2,4),
		Target = Point(2,2),
		Enemy = Point(2,2),
		Enemy2 = Point(2,1),
		Enemy3 = Point(3,2),
		Enemy4 = Point(1,2),
		Enemy5 = Point(2,3)
	}
}
					
function Vortex_Artillery:GetSkillEffect(p1,p2)
	local ret = SkillEffect()
	local dir = GetDirection(p2 - p1)
	local dir2 = GetDirection(p1 - p2)
	local damage = SpaceDamage(p2, self.Damage)
	if self.CentreSmoke then
		damage.iSmoke = EFFECT_CREATE
	end
	
	--Centre--
	ret:AddBounce(p1, -2)
	damage.sAnimation = "airpush_"..(dir)
	ret:AddArtillery(damage,"effects/shotup_ice.png")
	ret:AddBounce(p2, self.BounceAmount)
	
	--[0,1]--
	damage = SpaceDamage(p2 + DIR_VECTORS[dir], self.DamageOuter, (dir+1)%4)
	damage.sAnimation = "airpush_"..((dir+1)%4)
	ret:AddDamage(damage)
	
	--[1,0]--
	damage = SpaceDamage(p2 + DIR_VECTORS[(dir+1)%4], self.DamageOuter, dir2)
	damage.sAnimation = "airpush_"..(dir2)
	ret:AddDamage(damage)	
	
	--[0,-1]--
	damage = SpaceDamage(p2 - DIR_VECTORS[dir], self.DamageOuter, (dir-1)%4)
	damage.sAnimation = "airpush_"..((dir-1)%4)
	ret:AddDamage(damage)
	
	--[-1,0]--
	damage = SpaceDamage(p2 + DIR_VECTORS[(dir-1)%4], self.DamageOuter, dir)
	damage.sAnimation = "airpush_"..(dir)
	ret:AddDamage(damage)	
	
	return ret
end

Vortex_Artillery_A = Vortex_Artillery:new{
	UpgradeDescription = "Increases the outer ring damage by 2.",
	DamageOuter = 2
}

Vortex_Artillery_B = Vortex_Artillery:new{
	UpgradeDescription = "Creates smoke on the centre tile.",
	CentreSmoke = true
}

Vortex_Artillery_AB = Vortex_Artillery:new{
	DamageOuter = 2,
	CentreSmoke = true
}
